/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "../../libs.h"
#include "../../display/camera.h"
#include "../../display/font.h"
#include "../../display/shader/shader.h"
#include "../../display/primitive/primitive.h"
#include "../../display/primitive/quad.h"
#include "../../display/primitive/cube.h"
#include "../../display/scaledtexture.h"
#include "../../file/asset.h"
#include "../../ui/label.h"
#include "../../ui/breakpoint.h"
#include "../../file/assetmanager.h"
#include "../../file/xml.h"
#include "../../engine/engine.h"
#include "../../display/renderlist.h"
#include "../scene/scene.h"

typedef struct {
  engine_t engine; 
  scene_t scene;
  primitive_t quad;

  assetmanageritem_t *shader;
  assetmanageritem_t *shader2;
  assetmanageritem_t *scaledTexture;
  assetmanageritem_t *textureScale;
} sandboxgame_t;

/**
 * Initialize the sandbox scene test game.
 * 
 * @param game Game to initialize.
 * @param engine Engine to use during init.
 * @return True if successful, otherwise false.
 */
bool sandboxGameInit(sandboxgame_t *game);

/**
 * Update a sandbox scene.
 * 
 * @param game Game to update.
 * @param engine Engine to use when updating.
 */
void sandboxGameUpdate(sandboxgame_t *game);

/**
 * Dispose a previously created scene game.
 * 
 * @param game Game to dispose.
 */
void sandboxGameDispose(sandboxgame_t *game);