/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "../libs.h"
#include "../file/assetmanager.h"
#include "../engine/engine.h"
#include "../display/renderlist.h"
#include "../display/shader/shader.h"

// Forwarder for scene_t
typedef struct _scene_t scene_t;

// Callbacks
typedef void scenecallback_t(scene_t *scene);

// Scene Definition
typedef struct _scene_t {
  engine_t *engine;
  assetmanagerowner_t assetOwner;
  renderlist_t renderList;
  void *user;

  bool loaded;

  /** Scene Callbacks */
  scenecallback_t *onLoaded;
  scenecallback_t *onRender;
} scene_t;

void sceneInit(scene_t *scene, engine_t *engine, void *user);


float sceneUpdate(
  scene_t *scene, int32_t width, int32_t height, shader_t *shader,
  shaderuniform_t *uniforms
);

void sceneUpdateToBackbuffer(scene_t *scene,
  shader_t *shaderRenderList, shaderuniform_t *listUniforms,
  shader_t *shaderBackBuffer, shaderuniform_t *backUniforms
);

void sceneDispose(scene_t *scene);

renderpass_t * sceneUsePass(scene_t *scene, uint8_t i);